@@ -71,6 +71,7 @@ init.sts <- function(.Object, epoch, start=c(2000,1), freq=52, observed, state=0
 
     if (is.null(neighbourhood))
       neighbourhood <- matrix(NA,nrow=ncol(observed),ncol=ncol(observed))
+      #diag(neighbourhood) <- 0  #FIXME: shouldn't we always define the diag as 0?
     if (is.null(alarm)) 
       alarm      <- matrix(NA,nrow=dim(observed)[1],ncol=dim(observed)[2])
     if (is.null(upperbound))
@@ -204,6 +205,7 @@ setMethod("aggregate", signature(x="sts"), function(x,by="time",nfreq="all",...)
     x@upperbound <- matrix(NA,ncol=ncol(x@alarm),nrow=nrow(x@alarm))
     x@populationFrac <- as.matrix(apply(x@populationFrac, MARGIN=1, sum))#>0
     x@neighbourhood <- matrix(1,nrow=1,ncol=1)
+                     # FIXME: |-wouldn't 0 be more appropriate?
   }
 
   #validObject(x) #just a check
@@ -285,7 +287,10 @@ setMethod("[", "sts", function(x, i, j, ..., drop) {
 
   #Save time by not allocating a new object
   #res <- new("sts",epoch=week, freq=x@freq, start=start,observed=observed,state=state,alarm=alarm,upperbound=upperbound,neighbourhood=neighbourhood,populationFrac=populationFrac,map=x@map,control=x@control)
-    
+
+  ## FIXME: also subset map according to region index j? Here's the code:
+  ##x@map <- x@map[colnames(x@observed),]
+  
   return(x)
 })
 
@@ -839,15 +844,16 @@ wait <- function(wait.ms) {
 }
 
 #Helper function -- use colors from the vcd package
-hcl.colors <- function(x,ncolors=100,use.color=TRUE) {
-  if (use.color) {
-    #The Zeil-ice colors 
-    GYR <- rev(colorspace::heat_hcl(ncolors, h=c(0,120), c=c(90,30), l=c(50,90), power=c(0.75, 1.2)))
-  } else {
-    #Sanity check
-    GYR <- rev(colorspace::heat_hcl(ncolors, h=c(0,120), c=0, l=c(50,90), power=c(0.75, 1.2)))
-  }
-  return(list(col=GYR,min=0,max=max(x), trans=function(x) return(x)))
+hcl.colors <- function(x, ncolors=100, use.color=TRUE)
+{
+    GYR <- if (require("colorspace")) { # the Zeil-ice colors 
+        colorspace::heat_hcl(ncolors, h=c(0,120),
+                             c=if (use.color) c(90,30) else c(0,0),
+                             l=c(50,90), power=c(0.75, 1.2))
+    } else {
+        if (use.color) heat.colors(ncolors) else grey.colors(ncolors)
+    }
+    list(col=rev(GYR), min=0, max=max(x), trans=base::identity)
 }
 
 #Helper function for plot.sts.spacetime
@@ -913,6 +919,9 @@ setValidity("sts", function ( object ) {
   if (!all( colnames(object@observed) == colnames(object@neighbourhood)))
     retval <- c( retval , " colnames of observed and neighbourhood have to match")
 
+  ## FIXME: if map slot is not NULL, check that all colnames(object@observed)
+  ## are in row.names(object@map) (ideally, these are identical sets)
+  
   if(is.null( retval)) return ( TRUE )
   else return ( retval )
 })
