@@ -1,17 +1,22 @@
 ################################################################################
-### Function 'twinstim' performs maximum likelihood inference
-### for the additive-multiplicative spatio-temporal intensity model.
-### It uses 'nlminb' as the default optimizer (Newton-algorithm).
+### Part of the surveillance package, http://surveillance.r-forge.r-project.org
+### Free software under the terms of the GNU General Public License, version 2,
+### a copy of which is available at http://www.r-project.org/Licenses/.
 ###
-### Author: Sebastian Meyer
-### $Date: 2012-08-17 17:48:09 +0200 (Fr, 17. Aug 2012) $
+### Maximum Likelihood inference for the two-component spatio-temporal intensity
+### model described in Meyer et al (2012), DOI: 10.1111/j.1541-0420.2011.01684.x
+###
+### Copyright (C) 2012 Sebastian Meyer
+### $Revision: 454 $
+### $Date: 2012-11-15 15:43:56 +0100 (Do, 15. Nov 2012) $
 ################################################################################
 
 
 twinstim <- function (endemic, epidemic, siaf, tiaf, qmatrix = data$qmatrix,
     data, subset, t0 = data$stgrid$start[1], T = tail(data$stgrid$stop,1),
-    na.action = na.fail, nCub, nCub.adaptive = FALSE, partial = FALSE,
-    optim.args, finetune = FALSE, model = FALSE, cumCIF = TRUE, cumCIF.pb = TRUE)
+    na.action = na.fail, partial = FALSE,
+    control.siaf = list(F=list(), Deriv=list()), optim.args, finetune = FALSE,
+    model = FALSE, cumCIF = TRUE, cumCIF.pb = TRUE)
 {
 
     ####################
@@ -39,8 +44,9 @@ twinstim <- function (endemic, epidemic, siaf, tiaf, qmatrix = data$qmatrix,
         eventDists, eventsData, finetune, neghess, fisherinfo, fit, fixed,
         functions, globalEndemicIntercept, inmfe, initpars, ll, negll, loglik,
         mfe, mfhEvents, mfhGrid, model, my.na.action, na.action, namesOptimUser,
-        namesOptimArgs, nlminbControl, nlminbRes, nmRes, optim.args, optimArgs, 
-        optimControl, optimMethod, optimRes, optimRes1, optimValid, partial,
+        namesOptimArgs, nlminbControl, nlminbRes, nlmObjective, nlmControl,
+        nlmRes, nmRes, optim.args, optimArgs, control.siaf,
+        optimMethod, optimRes, optimRes1, optimValid, partial,
         partialloglik, ptm, qmatrix, res, negsc, score, subset, tmpexpr,
         typeSpecificEndemicIntercept, useScore, whichfixed, 
         inherits = FALSE)))
@@ -135,6 +141,8 @@ twinstim <- function (endemic, epidemic, siaf, tiaf, qmatrix = data$qmatrix,
         xx
     }
 
+    ID <- tile <- type <- BLOCK <- .obsInfLength <- .bdist <-
+        "just cheating on codetools::checkUsage"
     mfe <- model.frame(epidemic, data = eventsData,
                        subset = time + eps.t > t0 & time <= T,
 # here we can have some additional rows (individuals) compared to mfhEvents, which is established below!
@@ -145,7 +153,7 @@ twinstim <- function (endemic, epidemic, siaf, tiaf, qmatrix = data$qmatrix,
                        ID = ID, time = time, tile = tile, type = type,
                        eps.t = eps.t, eps.s = eps.s, BLOCK = BLOCK,
                        obsInfLength = .obsInfLength, bdist = .bdist)
-
+    rm(ID, tile, type, BLOCK, .obsInfLength, .bdist)
 
     ### Extract essential information from model frame
 
@@ -240,12 +248,14 @@ twinstim <- function (endemic, epidemic, siaf, tiaf, qmatrix = data$qmatrix,
 
     ### Generate endemic model frame and model matrix on event data
 
+    ID <- "just cheating on codetools::checkUsage"
     mfhEvents <- model.frame(endemic, data = eventsData,
                              subset = time>t0 & time<=T & ID %in% mfe[["(ID)"]],
                              na.action = na.fail,
                              # since R 2.10.0 patched also works with
                              # endemic = ~1 (see PR#14066)
                              drop.unused.levels = FALSE)
+    rm(ID)
     mmhEvents <- model.matrix(endemic, mfhEvents)
     # exclude intercept from endemic model matrix below, will be treated separately
     if (nbeta0 > 0) mmhEvents <- mmhEvents[,-1,drop=FALSE]
@@ -258,15 +268,17 @@ twinstim <- function (endemic, epidemic, siaf, tiaf, qmatrix = data$qmatrix,
 
     if (hash) {
         offsetEvents <- model.offset(mfhEvents)
+        BLOCK <- tile <- area <- "just cheating on codetools::checkUsage"
         mfhGrid <- model.frame(endemic, data = data$stgrid,
                                subset = start >= t0 & stop <= T,
                                na.action = na.fail,
                                # since R 2.10.0 patched also works with
                                # endemic = ~1 (see PR#14066)
                                drop.unused.levels = FALSE,
-                               BLOCK = BLOCK, tile = tile, dt = stop-start, ds = area)
-                               # 'tile' is redundant here for fitting, but is
-                               # useful for debugging and necessary for post. intensityplots
+                               BLOCK=BLOCK, tile=tile, dt=stop-start, ds=area)
+                               # 'tile' is redundant here for fitting but useful
+                               # for debugging & necessary for intensityplots
+        rm(BLOCK, tile, area)
         gridBlocks <- mfhGrid[["(BLOCK)"]]
         histIntervals <- unique(data$stgrid[c("BLOCK", "start", "stop")]) # sorted
         rownames(histIntervals) <- NULL
@@ -310,25 +322,30 @@ twinstim <- function (endemic, epidemic, siaf, tiaf, qmatrix = data$qmatrix,
         constanttiaf <- attr(tiaf, "constant")
         ntiafpars <- tiaf$npars
 
+        ## Check control.siaf
+        if (constantsiaf) control.siaf <- NULL else {
+            stopifnot(is.null(control.siaf) || is.list(control.siaf))
+            if (is.list(control.siaf)) stopifnot(sapply(control.siaf, is.list))
+        }
+        
         ## Define function that integrates the 'tiaf' function
         .tiafInt <- .tiafIntFUN()
 
         ## Define function that integrates the two-dimensional 'siaf' function
         ## over the influence regions of the events
-        .siafInt <- .siafIntFUN(siaf = siaf, nCub.adaptive = nCub.adaptive,
-                                noCircularIR = all(eps.s > bdist))
-        ## CAVE: nCub or nCub.adaptive might have been fixed by the above call
+        .siafInt <- .siafIntFUN(siaf = siaf, noCircularIR = all(eps.s > bdist))
+        .siafInt.args <- c(alist(siafpars), control.siaf$F)
 
-        ## Check nCub
-        if (!constantsiaf) {
-            stopifnot(is.vector(nCub, mode="numeric"))
-            if (any(is.na(nCub) | nCub <= 0L)) {
-                stop("'nCub' must be positive")
-            }
-            if (isTRUE(cl[["nCub.adaptive"]]) && !nCub.adaptive) {
-                message("'nCub.adaptive' only works in conjunction with ",
-                        "specified 'siaf$effRange()'")
-            }
+        ## Memoisation of .siafInt
+        ..siafInt <- if (requireNamespace("memoise")) {
+            memoise::memoise(.siafInt)
+            ## => speed-up optimization since 'nlminb' evaluates the loglik and
+            ## score for the same set of parameters at the end of each iteration
+        } else {
+            cat("Continuing without memoisation of 'siaf$f' cubature ...\n")
+            ## However, trivial caching is used manually in case of fixed
+            ## "siafpars" and in LambdagEvents()
+            .siafInt
         }
 
     } else {
@@ -339,7 +356,7 @@ twinstim <- function (endemic, epidemic, siaf, tiaf, qmatrix = data$qmatrix,
             warning("'tiaf' can only be modelled in conjunction with an 'epidemic' process")
         siaf <- tiaf <- NULL
         nsiafpars <- ntiafpars <- 0L
-        nCub <- nCub.adaptive <- NULL
+        control.siaf <- NULL
 
     }
 
@@ -352,7 +369,19 @@ twinstim <- function (endemic, epidemic, siaf, tiaf, qmatrix = data$qmatrix,
         (!hastiafpars | (!is.null(tiaf$deriv)) & !is.null(tiaf$Deriv))
     } else TRUE
 
-
+    ## Define function that applies siaf$Deriv on all events (integrate the
+    ## two-dimensional siaf$deriv function)
+    if (useScore && hassiafpars) {
+        .siafDeriv <- function (siafpars, ...) {
+            derivInti <- function (i)
+                siaf$Deriv(influenceRegion[[i]], siaf$deriv, siafpars,
+                           eventTypes[i], ...)
+            derivInt <- sapply(1:N, derivInti)
+            #<- N-vector or nsiafpars x N matrix => transform to N x nsiafpars
+            if (is.matrix(derivInt)) t(derivInt) else as.matrix(derivInt)
+        }
+        .siafDeriv.args <- c(alist(siafpars), control.siaf$Deriv)
+    }
 
 
 
@@ -467,16 +496,16 @@ twinstim <- function (endemic, epidemic, siaf, tiaf, qmatrix = data$qmatrix,
                           uppert = NULL) {}
     body(heIntTWK) <- as.call(c(as.name("{"),
         if (hash) { # endemic component
-            expression({
-                hIntTW <- .hIntTW(beta, uppert = uppert)
-                .beta0 <- rep(if (nbeta0==0L) 0 else beta0, length.out = nTypes)
-                fact <- sum(exp(.beta0))
+            expression(
+                hIntTW <- .hIntTW(beta, uppert = uppert),
+                .beta0 <- rep(if (nbeta0==0L) 0 else beta0, length.out = nTypes),
+                fact <- sum(exp(.beta0)),
                 hInt <- fact * hIntTW
-            })
+            )
         } else { expression(hInt <- 0) },
         if (hase) { # epidemic component
-            expression({
-                siafInt <- .siafInt(siafpars) # N-vector
+            expression(
+                siafInt <- do.call("..siafInt", .siafInt.args), # N-vector
                 if (!is.null(uppert)) { # && isScalar(uppert) && t0 <= uppert && uppert < T
                     gIntUpper <- pmin(uppert-eventTimes, eps.t)
                     subtimeidx <- eventTimes < uppert
@@ -490,7 +519,7 @@ twinstim <- function (endemic, epidemic, siaf, tiaf, qmatrix = data$qmatrix,
                     tiafInt <- .tiafInt(tiafpars)
                     eInt <- sum(qSum * gammapred * siafInt * tiafInt)
                 }
-            })
+            )
         } else expression(eInt <- 0),
         expression(c(hInt, eInt))
     ))
@@ -543,7 +572,7 @@ twinstim <- function (endemic, epidemic, siaf, tiaf, qmatrix = data$qmatrix,
             hEvents <- if (hash) .hEvents(beta0, beta) else 0
             eEvents <- .eEvents(gammapred, siafpars, tiafpars) # Nin-vector! (only 'includes' here)
             lambdaEvents <- hEvents + eEvents  # Nin-vector
-            siafInt <- .siafInt(siafpars) # N-vector
+            siafInt <- do.call("..siafInt", .siafInt.args) # N-vector
             tiafInt <- .tiafInt(tiafpars) # N-vector
         }
 
@@ -595,18 +624,7 @@ twinstim <- function (endemic, epidemic, siaf, tiaf, qmatrix = data$qmatrix,
                 nom <- .eEvents(gammapred, siafpars, tiafpars,
                                 ncolsRes=nsiafpars, f=siaf$deriv) # Nin x nsiafpars matrix
                 sEventsSum <- colSums(nom / lambdaEvents)
-                epsTypes <- if (nCub.adaptive) {
-                        siaf$effRange(siafpars) / nCub
-                        ## FIXME: this bandwidth is actually not adapted for siaf$deriv
-                    } else nCub
-                epsTypes <- rep(epsTypes, length.out = nTypes)
-                derivInt <- sapply(1:nsiafpars, function (paridx) {
-                    sapply(1:N, function (i) {
-                        polyCub.midpoint(influenceRegion[[i]], function (s) {
-                            siaf$deriv(s, siafpars, eventTypes[i])[,paridx,drop=TRUE]
-                        }, eps = epsTypes[eventTypes[i]])
-                    })
-                })  # Nxnsiafpars matrix
+                derivInt <- do.call(".siafDeriv", .siafDeriv.args) # N x nsiafpars matrix
                 sInt <- colSums(derivInt * (qSum * gammapred * tiafInt))
                 sEventsSum - sInt
             }) else numeric(nsiafpars) # if 'fixedsiafpars', this part is unused
@@ -741,7 +759,7 @@ twinstim <- function (endemic, epidemic, siaf, tiaf, qmatrix = data$qmatrix,
                 unname(hInt_blocks[.idx])   # Nin-vector
             } else 0
         eInts <- if (hase) { # epidemic component
-                siafInt <- .siafInt(siafpars) # N-vector
+                siafInt <- do.call(".siafInt", .siafInt.args) # N-vector
                 gs <- gammapred * siafInt # N-vector
                 sapply(includes, function (i) {
                     timeSources <- determineSources(i, eventTimes, removalTimes,
@@ -785,7 +803,8 @@ twinstim <- function (endemic, epidemic, siaf, tiaf, qmatrix = data$qmatrix,
         body(ll) <- as.call(append(as.list(body(ll)),
             as.list(expression(
                 if (hassiafpars && !siaf$validpars(siafpars)) {
-                    if (optimArgs$control$trace > 0L) cat("(invalid 'siafpars' in loglik)\n")
+                    if (optimArgs$control$trace > 0L)
+                        cat("(invalid 'siafpars' in loglik)\n")
                     return(-Inf)
                 }
                 )),
@@ -852,40 +871,6 @@ twinstim <- function (endemic, epidemic, siaf, tiaf, qmatrix = data$qmatrix,
         if (hastiafpars) paste("e.tiaf",1:ntiafpars,sep=".")
     )
 
-    
-    ### Message about potentially cumbersome numerical cubature
-    
-    if (hassiafpars && !is.null(siaf$Fcircle)) {
-        .maxnpixels <- local({
-            if (nCub.adaptive) {
-                initsiaf <- optim.args$par[grep("^e\\.siaf", names(optim.args$par))]
-                initeffRangeTypes <- rep(siaf$effRange(initsiaf),length.out=nTypes)
-                initeffRanges <- initeffRangeTypes[eventTypes]
-                border <- which(eps.s > bdist & initeffRanges > bdist) # here we integrate
-                if (length(border) > 0L) {
-                    maxarea <- spatstat::bounding.box(influenceRegion[border][[which.max(iRareas[border])]])
-                    inithmin <- min(initeffRangeTypes) / nCub
-                    ceiling(c(diff(maxarea$xrange), diff(maxarea$yrange)) / inithmin)
-                } else NULL
-            } else {
-                border <- which(eps.s > bdist) # for these events we must integrate
-                if (length(border) > 0L) {
-                    maxarea <- spatstat::bounding.box(influenceRegion[border][[which.max(iRareas[border])]])
-                    ceiling(c(diff(maxarea$xrange), diff(maxarea$yrange)) / nCub)
-                } else NULL
-            }
-        })
-        if (!is.null(.maxnpixels) && prod(.maxnpixels) > 10000) {
-            cat("\nNOTE: the initial values of the 'e.siaf' parameters potentially require up to",
-                "\n     ", .maxnpixels[1], "x", .maxnpixels[2], "pixels for the midpoint cubature.",
-                "If iterations are slow,",
-                "\n      consider increasing",
-                if (nCub.adaptive) {
-                    "'eps = siaf$effRange(siafpars) / nCub'."
-                } else "'nCub'.", "\n\n")
-        }
-    }
-
 
     ### Fixed parameters during optimization
 
@@ -955,7 +940,8 @@ twinstim <- function (endemic, epidemic, siaf, tiaf, qmatrix = data$qmatrix,
         ## if siafpars or tiafpars are fixed, pre-evaluate integrals    
         if (fixedsiafpars) {
             cat("pre-evaluating 'siaf' integrals with fixed parameters ...\n")
-            siafInt <- .siafInt(initpars[paste("e.siaf", 1:nsiafpars, sep=".")])
+            .siafInt.args[[1]] <- initpars[paste("e.siaf", 1:nsiafpars, sep=".")]
+            siafInt <- do.call(".siafInt", .siafInt.args)
             ## re-define .siafInt such that it just returns the pre-evaluated integrals
             .siafInt.orig <- .siafInt
             body(.siafInt) <- expression(siafInt)
@@ -994,16 +980,11 @@ twinstim <- function (endemic, epidemic, siaf, tiaf, qmatrix = data$qmatrix,
 
         ## Configure the optim procedure (check optim.args)
 
-        # default control arguments
-        optimControl <- list(trace = 1L, REPORT = 5L)
-        # merge with user control arguments
-        optimControl[names(optim.args[["control"]])] <- optim.args[["control"]]
-        optim.args$control <- optimControl
         # default arguments
         optimArgs <- alist(par =, fn = negll, gr = negsc,
                            method = if (partial) "Nelder-Mead" else "nlminb",
                            lower = -Inf, upper = Inf,
-                           control = list(), hessian = partial | !useScore)
+                           control = list(), hessian = TRUE)
         # user arguments
         namesOptimArgs <- names(optimArgs)
         namesOptimUser <- names(optim.args)
@@ -1017,39 +998,68 @@ twinstim <- function (endemic, epidemic, siaf, tiaf, qmatrix = data$qmatrix,
         optimMethod <- eval(optimArgs$method)
 
 
-        ## Call 'optim' or 'nlminb' (default) with the above arguments
+        ## Call 'optim', 'nlminb', or 'nlm' with the above arguments
 
         cat("\nminimizing the negative", if (partial) "partial", "log-likelihood",
-            "using", if (optimMethod == "nlminb") "'nlminb()'" else {
+            "using", if (optimMethod %in% c("nlm", "nlminb"))
+            paste0("'",optimMethod,"()'") else {
                 paste0("'optim()'s \"", optimMethod, "\"")
             }, "...\n")
         cat("initial parameters:\n")
         print(optimArgs$par)
         optimRes1 <- if (optimMethod == "nlminb") {
-            nlminbControl <- optimArgs$control[c("maxit","REPORT","abstol","reltol")]
-            names(nlminbControl) <- c("iter.max", "trace", "abs.tol", "rel.tol")
-            if (optimArgs$control$trace == 0L) nlminbControl$trace <- 0L
+            nlminbControl <- control2nlminb(optimArgs$control,
+                                            defaults = list(trace=5L, rel.tol=1e-6))
+            ## sqrt(.Machine$double.eps) is the default reltol used in optim,
+            ## which usually equals about 1.49e-08.
+            ## The default rel.tol of nlminb (1e-10) seems too small
+            ## (nlminb often does not finish despite no "relevant" change in loglik).
+            ## I therefore use 1e-6, which is also the default in package nlme
+            ## (see 'lmeControl').
             if (nlminbControl$trace > 0L) {
                 cat("negative log-likelihood and parameters ")
                 if (nlminbControl$trace == 1L) cat("in each iteration") else {
                     cat("every", nlminbControl$trace, "iterations") }
                 cat(":\n")
             }
-            if (is.null(nlminbControl$rel.tol)) nlminbControl$rel.tol <- 1e-6
-            ## sqrt(.Machine$double.eps) is the default reltol used in optim,
-            ## which usually equals about 1.49e-08.
-            ## The default rel.tol of nlminb (1e-10) seems too small
-            ## (nlminb often does not finish despite no "relevant" change in loglik).
-            ## I therefore use 1e-6, which is also the default in package nlme (see lmeControl).
-            nlminbControl <- nlminbControl[!sapply(nlminbControl, is.null)]
             nlminbRes <- nlminb(start = optimArgs$par, objective = negll,
-                                gradient = negsc, hessian = neghess,
+                                gradient = negsc,
+                                hessian = if (doHessian) neghess else NULL,
                                 control = nlminbControl,
                                 lower = optimArgs$lower, upper = optimArgs$upper)
             nlminbRes$value <- -nlminbRes$objective
             nlminbRes$counts <- nlminbRes$evaluations
             nlminbRes
-        } else {
+        } else if (optimMethod == "nlm") {
+            nlmObjective <- function (theta) {
+                value <- negll(theta)
+                grad <- negsc(theta)
+                #hess <- neghess(theta)
+                structure(value, gradient = grad)#, hessian = hess)
+            }
+            nlmControl <- optimArgs$control
+            if (is.null(nlmControl[["print.level"]])) {
+                nlmControl$print.level <- min(nlmControl$trace, 2L)
+            }
+            nlmControl$trace <- nlmControl$REPORT <- NULL
+            if (is.null(nlmControl[["iterlim"]])) {
+                nlmControl$iterlim <- nlmControl$maxit
+            }
+            nlmControl$maxit <- NULL
+            nlmControl$check.analyticals <- FALSE
+            ##<- we use the negative _expected_ Fisher information as the Hessian,
+            ##   which is of course different from the true Hessian (=neg. obs. Fisher info)
+            nlmRes <- do.call("nlm", c(alist(f = nlmObjective, p = optimArgs$par,
+                                             hessian = doHessian),
+                                             nlmControl))
+            names(nlmRes)[names(nlmRes) == "estimate"] <- "par"
+            nlmRes$value <- -nlmRes$minimum
+            nlmRes$counts <- rep.int(nlmRes$iterations, 2L)
+            nlmRes$convergence <- if (nlmRes$code %in% 1:2) 0L else nlmRes$code
+            nlmRes
+        } else { # use optim()
+            optimArgs$control <- modifyList(list(trace=1L, REPORT=5L),
+                                            optimArgs$control)
             if (finetune) optimArgs$hessian <- FALSE
             res <- do.call("optim", optimArgs)
             res$value <- -res$value
@@ -1067,6 +1077,7 @@ twinstim <- function (endemic, epidemic, siaf, tiaf, qmatrix = data$qmatrix,
             optimArgs$par <- optimRes1$par
             optimArgs$method <- "Nelder-Mead"
             optimArgs$hessian <- doHessian
+            optimArgs$control <- modifyList(list(trace=1L), optimArgs$control)
             nmRes <- do.call("optim", optimArgs)
             nmRes$value <- -nmRes$value
             nmRes$counts[2L] <- 0L   # 0 gradient evaluations (replace NA for addition below)
@@ -1075,9 +1086,10 @@ twinstim <- function (endemic, epidemic, siaf, tiaf, qmatrix = data$qmatrix,
 
         if (optimRes$convergence != 0) {
             cat("\nWARNING: OPTIMIZATION ROUTINE DID NOT CONVERGE",
-                if (optimMethod != "nlminb") paste0("(code ", optimRes$convergence, ")"),
+                if (finetune || optimMethod != "nlminb")
+                paste0("(code ", optimRes$convergence, ")"),
                 "!\n")
-            if (!is.null(optimRes$message) && nchar(optimRes$message) > 0L) {
+            if (!is.null(optimRes$message) && nzchar(optimRes$message)) {
                 cat("MESSAGE: \"", optimRes$message, "\"\n", sep="")
             }
         }
@@ -1113,11 +1125,8 @@ twinstim <- function (endemic, epidemic, siaf, tiaf, qmatrix = data$qmatrix,
            loglik = structure(if (all(fixed)) ll(initpars) else optimRes$value,
                               partial = partial),
            counts = if (all(fixed)) c("function"=1L, "gradient"=0L) else {
-               optimRes1$counts + if (finetune) optimRes$counts else 0L
+               optimRes1$counts + if (finetune) optimRes$counts else c(0L, 0L)
            },
-           method = if (all(fixed)) NA_character_ else optimMethod,
-           nCub = nCub, nCub.adaptive = nCub.adaptive,
-           ## nCub and nCub.adaptive are queried for default eps in R0.twinstim
            converged = all(fixed) || (optimRes$convergence == 0)
            )
 
@@ -1129,7 +1138,8 @@ twinstim <- function (endemic, epidemic, siaf, tiaf, qmatrix = data$qmatrix,
 
     # If requested, add observed fisher info (= negative hessian at maximum)
     if (any(!fixed) && !is.null(optimRes$hessian)) {
-        fit$fisherinfo.observed <- -optimRes$hessian
+        fit$fisherinfo.observed <- optimRes$hessian
+        ## no "-" here because we optimized the negative log-likelihood
     }
 
 
@@ -1147,7 +1157,7 @@ twinstim <- function (endemic, epidemic, siaf, tiaf, qmatrix = data$qmatrix,
     # and final gammapred (also used by intensity.twinstim)
     if (hase) {
         gammapred <- drop(exp(mme %*% gamma)) # N-vector
-        if (!fixedsiafpars) siafInt <- .siafInt(siafpars)
+        if (!fixedsiafpars) siafInt <- do.call("..siafInt", .siafInt.args)
         if (!fixedtiafpars) tiafInt <- .tiafInt(tiafpars)
     }
     
@@ -1167,7 +1177,7 @@ twinstim <- function (endemic, epidemic, siaf, tiaf, qmatrix = data$qmatrix,
         if (hase) {
             ## tiny hack such that siaf integrals are not evaluated N-fold
             .siafInt.orig <- .siafInt
-            .siafInt <<- function (siafpars) siafInt
+            body(.siafInt) <<- expression(siafInt)
             on.exit(.siafInt <<- .siafInt.orig)
         }
         heIntEvents <- matrix(NA_real_, Nin, 2L)
@@ -1200,7 +1210,7 @@ twinstim <- function (endemic, epidemic, siaf, tiaf, qmatrix = data$qmatrix,
     fit$npars <- c(nbeta0 = nbeta0, p = p,
                    q = q, nsiafpars = nsiafpars, ntiafpars = ntiafpars)
     fit$qmatrix <- qmatrix   # -> information about nTypes and typeNames
-    fit$bbox <- sp::bbox(data$W)        # for completeness and for iafplot
+    fit$bbox <- bbox(data$W)            # for completeness and for iafplot
     fit$timeRange <- c(t0, T)           # for simulate.twinstim's defaults
     if (!model) {
         # Link formulae to the global environment such that the evaluation
@@ -1215,6 +1225,12 @@ twinstim <- function (endemic, epidemic, siaf, tiaf, qmatrix = data$qmatrix,
                    epidemic = formula(epidemic),
                    siaf = siaf, tiaf = tiaf
                    )
+    fit$control.siaf <- control.siaf
+
+    
+    ### Append optimizer configuration
+    
+    fit$optim.args <- optim.args
     if (model) {
         fit$functions <- functions
         environment(fit) <- environment()
